@@ -46,7 +46,7 @@ class ReadOnlyCondition:
     def evaluate(self, settings: List["Setting"]) -> bool:
         for setting in settings:
             if setting.name.lower() == self.name.lower():
-                if setting.current in self.values:
+                if (setting.current if setting.current is not None else setting.default) in self.values:
                     return self.negate
                 else:
                     return not self.negate
@@ -56,6 +56,24 @@ class ReadOnlyCondition:
             self.filename,
         )
 
+    def __eq__(self, other: object) -> bool:
+        if other is self:
+            return True
+        if not isinstance(other, ReadOnlyCondition):
+            return False
+
+        return (
+            self.filename == other.filename and
+            self.setting == other.setting and
+            self.name == other.name and
+            self.values == other.values and
+            self.negate == other.negate
+        )
+
+    def __ne__(self, other: object) -> bool:
+        # Python documentation recommends doing this instead of explicitly calling __eq__.
+        return not self == other
+
 
 class DefaultCondition:
     # A wrapper class to encapsulate one rule for a setting default.
@@ -66,6 +84,23 @@ class DefaultCondition:
         self.negate = negate
         self.default = default
 
+    def __eq__(self, other: object) -> bool:
+        if other is self:
+            return True
+        if not isinstance(other, DefaultCondition):
+            return False
+
+        return (
+            self.name == other.name and
+            self.values == other.values and
+            self.negate == other.negate and
+            self.default == other.default
+        )
+
+    def __ne__(self, other: object) -> bool:
+        # Python documentation recommends doing this instead of explicitly calling __eq__.
+        return not self == other
+
 
 class DefaultConditionGroup:
     # A wrapper class to encapsulate a set of rules for defaulting a setting.
@@ -75,7 +110,23 @@ class DefaultConditionGroup:
         self.setting = setting
         self.conditions = conditions
 
-    def evaluate(self, filename: str, name: str, settings: List["Setting"]) -> int:
+    def __eq__(self, other: object) -> bool:
+        if other is self:
+            return True
+        if not isinstance(other, DefaultConditionGroup):
+            return False
+
+        return (
+            self.filename == other.filename and
+            self.setting == other.setting and
+            self.conditions == other.conditions
+        )
+
+    def __ne__(self, other: object) -> bool:
+        # Python documentation recommends doing this instead of explicitly calling __eq__.
+        return not self == other
+
+    def evaluate(self, settings: List["Setting"]) -> int:
         for cond in self.conditions:
             for setting in settings:
                 if setting.name.lower() == cond.name.lower():
@@ -331,17 +382,23 @@ class Setting:
 class Settings:
     # A collection of settings as well as the type of settings this is. This is also responsible
     # for parsing and creating sections in an actual EEPROM file based on the settings themselves.
+    # If your settings doesn't have a type (for instance if it is the only collection of settings
+    # for a given file format), you can leave this unset. Default endianness is LE, but if you want
+    # big endian, you can specify that as well.
 
-    def __init__(self, filename: str, settings: List[Setting], type: Optional[str] = None) -> None:
+    def __init__(self, filename: str, settings: List[Setting], type: Optional[str] = None, big_endian: bool = False) -> None:
         self.filename = filename
         self.settings = settings
         self.type = type
+        self.endian = ">" if big_endian else "<"
 
     @staticmethod
-    def from_config(type: str, config: "SettingsConfig", data: bytes) -> "Settings":
+    def from_config(config: "SettingsConfig", data: bytes, type: Optional[str] = None, big_endian: bool = False) -> "Settings":
         # Keep track of how many bytes into the EEPROM we are.
         location = 0
         halves = 0
+        halfname: Optional[str] = None
+        endian = ">" if big_endian else "<"
 
         # We need to make sure this is in load order, so that we can parse out the
         # correct settings in order of them showing up in the file.
@@ -354,54 +411,75 @@ class Settings:
                         setting.current = data[location] & 0xF
 
                 if halves == 0:
+                    halfname = setting.name
                     halves = 1
                 else:
+                    halfname = None
                     halves = 0
                     location += 1
             elif setting.size == SettingSizeEnum.BYTE:
                 if halves != 0:
-                    raise SettingsParseException(f"The setting \"{setting.name}\" follows a lonesome half-byte. Half-byte settings must always be in pairs!", config.filename)
+                    raise SettingsParseException(
+                        f"The setting \"{setting.name}\" follows a lonesome half-byte setting \"{halfname}\". Half-byte settings must always be in pairs!",
+                        config.filename
+                    )
 
                 if setting.length == 1:
                     if location < len(data):
-                        setting.current = struct.unpack("<B", data[location:(location + 1)])[0]
+                        setting.current = struct.unpack(endian + "B", data[location:(location + 1)])[0]
                 elif setting.length == 2:
                     if location < (len(data) - 1):
-                        setting.current = struct.unpack("<H", data[location:(location + 2)])[0]
+                        setting.current = struct.unpack(endian + "H", data[location:(location + 2)])[0]
                 elif setting.length == 4:
                     if location < (len(data) - 3):
-                        setting.current = struct.unpack("<I", data[location:(location + 4)])[0]
+                        setting.current = struct.unpack(endian + "I", data[location:(location + 4)])[0]
                 else:
                     raise SettingsParseException(f"Cannot parse setting \"{setting.name}\" with unrecognized size \"{setting.length}\"!", config.filename)
 
                 location += setting.length
 
+        if halves != 0:
+            raise SettingsSaveException(
+                f"The setting \"{halfname}\" is a lonesome half-byte setting. Half-byte settings must always be in pairs!",
+                config.filename
+            )
+
         # Return this in the original order presented to us.
-        final_settings = Settings(config.filename, config.settings, type=type)
+        final_settings = Settings(config.filename, config.settings, type=type, big_endian=big_endian)
         if final_settings.length != len(data):
             raise SettingsParseException(
-                f"Unexpected final size of {type} section, expected {len(data)} bytes but definition file covers {final_settings.length} bytes!",
+                f"Unexpected final size of {type if type is not None else 'unnamed'} section, expected {len(data)} bytes but definition file covers {final_settings.length} bytes!",
                 config.filename,
             )
         return final_settings
 
     @property
     def length(self) -> int:
+        # Note that this is different than just len(self.settings), because some settings can take up
+        # more or less than a single byte. This calculates the length of the data in bytes that will
+        # be returned by to_bytes()
         halves = 0
         length = 0
+        halfname: Optional[str] = None
+
         for setting in sorted(self.settings, key=lambda setting: setting.order):
             if setting.size == SettingSizeEnum.NIBBLE:
                 # Update our length.
                 if halves == 0:
+                    halfname = setting.name
                     halves = 1
                 else:
+                    halfname = None
                     halves = 0
                     length += 1
 
             elif setting.size == SettingSizeEnum.BYTE:
                 # First, make sure we aren't in a pending nibble state.
                 if halves != 0:
-                    raise SettingsSaveException(f"The setting \"{setting.name}\" follows a lonesome half-byte. Half-byte settings must always be in pairs!", self.filename)
+                    raise SettingsSaveException(
+                        f"The setting \"{setting.name}\" follows a lonesome half-byte setting \"{halfname}\". Half-byte settings must always be in pairs!",
+                        self.filename
+                    )
 
                 if setting.length not in {1, 2, 4}:
                     raise SettingsSaveException(f"Cannot save setting \"{setting.name}\" with unrecognized size {setting.length}!", self.filename)
@@ -409,14 +487,27 @@ class Settings:
                 # Update our length.
                 length += setting.length
 
+        if halves != 0:
+            raise SettingsSaveException(
+                f"The setting \"{halfname}\" is a lonesome half-byte setting. Half-byte settings must always be in pairs!",
+                self.filename
+            )
+
         return length
 
     @staticmethod
-    def from_json(type: str, config: "SettingsConfig", jsondict: Dict[str, Any], context: List[str]) -> "Settings":
+    def from_json(
+        config: "SettingsConfig",
+        jsondict: Dict[str, Any],
+        context: List[str],
+        type: Optional[str] = None,
+        big_endian: bool = False,
+    ) -> "Settings":
         # First, parse out the keys that we know we need.
-        typestr = jsondict.get('type')
-        if not isinstance(typestr, str) or typestr != type:
-            raise JSONParseException(f"\"type\" key in JSON has invalid data \"{typestr}\"!", context)
+        if type is not None:
+            typestr = jsondict.get('type')
+            if not isinstance(typestr, str) or typestr != type:
+                raise JSONParseException(f"\"type\" key in JSON has invalid data \"{typestr}\"!", context)
         settings = jsondict.get('settings')
         if not isinstance(settings, list):
             raise JSONParseException(f"\"settings\" key in JSON has invalid data \"{settings}\"!", context)
@@ -424,7 +515,7 @@ class Settings:
         if not isinstance(filename, str) or filename != config.filename:
             if filename is None and config.filename == NO_FILE:
                 # This is an empty settings file, just return an empty config.
-                return Settings(config.filename, config.settings, type=type)
+                return Settings(config.filename, config.settings, type=type, big_endian=big_endian)
             raise JSONParseException(f"\"filename\" key in JSON has invalid data \"{filename}\"!", context)
 
         # Now, parse out the settings in the dict.
@@ -440,20 +531,23 @@ class Settings:
             else:
                 raise JSONParseException(f"Setting \"{setting.name}\" could not be found in JSON!", context)
 
-        return Settings(config.filename, config.settings, type=type)
+        return Settings(config.filename, config.settings, type=type, big_endian=big_endian)
 
     def to_json(self) -> Dict[str, Any]:
-        return {
-            'type': self.type,
+        jsondata = {
             'filename': self.filename if self.filename != NO_FILE else None,
             'settings': [
                 setting.to_json() for setting in self.settings
             ],
         }
+        if self.type is not None:
+            jsondata['type'] = self.type
+        return jsondata
 
     def to_bytes(self) -> bytes:
         pending: int = 0
         halves: int = 0
+        halfname: Optional[str] = None
         location: int = 0
         section: bytes = b''
 
@@ -465,7 +559,7 @@ class Settings:
                 default = setting.default
             elif isinstance(setting.default, DefaultConditionGroup):
                 # Must evaluate settings to figure out the default for this.
-                default = setting.default.evaluate(self.filename, setting.name, self.settings)
+                default = setting.default.evaluate(self.settings)
 
             # Now, figure out if we should defer to the default over the current value
             # (if it is read-only) or if we should use the current value.
@@ -495,21 +589,26 @@ class Settings:
                     else:
                         if len(section) != location:
                             raise Exception("Logic error!")
-                        section += struct.pack("<B", (value & 0xF) | pending)
+                        section += struct.pack(self.endian + "B", (value & 0xF) | pending)
                 else:
                     raise SettingsSaveException(f"Cannot save setting \"{setting.name}\" with a null value!", self.filename)
 
                 # Now, update our position.
                 if halves == 0:
+                    halfname = setting.name
                     halves = 1
                 else:
+                    halfname = None
                     halves = 0
                     location += 1
 
             elif setting.size == SettingSizeEnum.BYTE:
                 # First, make sure we aren't in a pending nibble state.
                 if halves != 0:
-                    raise SettingsSaveException(f"The setting \"{setting.name}\" follows a lonesome half-byte. Half-byte settings must always be in pairs!", self.filename)
+                    raise SettingsSaveException(
+                        f"The setting \"{setting.name}\" follows a lonesome half-byte setting \"{halfname}\". Half-byte settings must always be in pairs!",
+                        self.filename
+                    )
 
                 if setting.length not in {1, 2, 4}:
                     raise SettingsSaveException(f"Cannot save setting \"{setting.name}\" with unrecognized size {setting.length}!", self.filename)
@@ -518,17 +617,23 @@ class Settings:
                     if len(section) != location:
                         raise Exception("Logic error!")
                     if setting.length == 1:
-                        section += struct.pack("<B", value)
+                        section += struct.pack(self.endian + "B", value)
                     elif setting.length == 2:
-                        section += struct.pack("<H", value)
+                        section += struct.pack(self.endian + "H", value)
                     elif setting.length == 4:
-                        section += struct.pack("<I", value)
+                        section += struct.pack(self.endian + "I", value)
                 else:
                     raise SettingsSaveException(f"Cannot save setting \"{setting.name}\" with a null value!", self.filename)
 
                 # Now, update our position.
                 location += setting.length
 
+        if halves != 0:
+            raise SettingsSaveException(
+                f"The setting \"{halfname}\" is a lonesome half-byte setting. Half-byte settings must always be in pairs!",
+                self.filename
+            )
+
         return section
 
     def __str__(self) -> str:
@@ -912,7 +1017,7 @@ class SettingsConfig:
             if size == SettingSizeEnum.UNKNOWN:
                 raise SettingsParseException(f"Setting \"{name}\" is missing a size specifier!", filename)
             if read_only is not True and not values:
-                raise SettingsParseException(f"Setting \"{name}\" is missing any valid values!", filename)
+                raise SettingsParseException(f"Setting \"{name}\" is missing any valid values and isn't read-only!", filename)
 
             # Add it to either the current list of settings, or the queue of settings to insert when we're done.
             new_setting = Setting(
@@ -932,15 +1037,6 @@ class SettingsConfig:
             # Keep track of order
             order += 1
 
-        # Verify that nibbles come in pairs.
-        halves = 0
-        for setting in settings:
-            if setting.size == SettingSizeEnum.NIBBLE:
-                halves = 1 - halves
-            elif setting.size == SettingSizeEnum.BYTE:
-                if halves != 0:
-                    raise SettingsParseException(f"The setting \"{setting.name}\" follows a lonesome half-byte. Half-byte settings must always be in pairs!", filename)
-
         # Now, insert pending settings where they belong.
         while pending_insertions:
             # Search all pending insertions to see if we can add one of them.
@@ -973,11 +1069,33 @@ class SettingsConfig:
                     filename,
                 )
 
+        # Finally, verify that nibbles come in pairs, which was dependent on the original
+        # file-ordering, not in the dependent ordering we calculated here.
+        halves = 0
+        halfname: Optional[str] = None
+        for setting in sorted(settings, key=lambda setting: setting.order):
+            if setting.size == SettingSizeEnum.NIBBLE:
+                halves = 1 - halves
+                halfname = None if halves == 0 else setting.name
+            elif setting.size == SettingSizeEnum.BYTE:
+                if halves != 0:
+                    raise SettingsParseException(
+                        f"The setting \"{setting.name}\" follows a lonesome half-byte setting {halfname}. Half-byte settings must always be in pairs!",
+                        filename
+                    )
+        if halves != 0:
+            raise SettingsSaveException(
+                f"The setting \"{halfname}\" is a lonesome half-byte setting. Half-byte settings must always be in pairs!",
+                filename
+            )
+
         return SettingsConfig(filename, settings)
 
+    @property
     def defaults(self) -> bytes:
         pending = 0
         halves = 0
+        halfname: Optional[str] = None
         defaults: List[bytes] = []
 
         for setting in sorted(self.settings, key=lambda setting: setting.order):
@@ -987,7 +1105,7 @@ class SettingsConfig:
                 default = setting.default
             elif isinstance(setting.default, DefaultConditionGroup):
                 # Must evaluate settings to figure out the default for this.
-                default = setting.default.evaluate(self.filename, setting.name, self.settings)
+                default = setting.default.evaluate(self.settings)
 
             if setting.size == SettingSizeEnum.NIBBLE:
                 if halves == 0:
@@ -996,12 +1114,17 @@ class SettingsConfig:
                     defaults.append(bytes([(default & 0xF) | pending]))
 
                 if halves == 0:
+                    halfname = setting.name
                     halves = 1
                 else:
+                    halfname = None
                     halves = 0
             elif setting.size == SettingSizeEnum.BYTE:
                 if halves != 0:
-                    raise SettingsSaveException(f"The setting \"{setting.name}\" follows a lonesome half-byte. Half-byte settings must always be in pairs!", self.filename)
+                    raise SettingsSaveException(
+                        f"The setting \"{setting.name}\" follows a lonesome half-byte setting \"{halfname}\". Half-byte settings must always be in pairs!",
+                        self.filename
+                    )
                 if setting.length == 1:
                     defaults.append(struct.pack("<B", default))
                 elif setting.length == 2:
@@ -1011,4 +1134,10 @@ class SettingsConfig:
                 else:
                     raise SettingsSaveException(f"Cannot save setting \"{setting.name}\" with unrecognized size {setting.length}!", self.filename)
 
+        if halves != 0:
+            raise SettingsSaveException(
+                f"The setting \"{halfname}\" is a lonesome half-byte setting. Half-byte settings must always be in pairs!",
+                self.filename
+            )
+
         return b"".join(defaults)
